## remove all previous variables and data
rm(list = ls())

## libraries.
library(MASS)
library(nlme)
library(splines)
library(survival)
library(randomForest)


## source the general function here with personalized directory
source("~\\...\\general_R_functions.R")

#########################################################
#                                                       #
#      Start Here                                       #
#                                                       #
#########################################################

## I.1, set parameters
n1           <- 200
n2           <- 1000
n_S          <- 4000
a_S          <- 0.005
B            <- 400
b            <- 0.1
lambda_up    <- 10
lambda_low   <- 0.2
delta_beta   <- 0.05
grid_number  <- 6
S_lambda     <- 6
outlier_max  <- 10
outlier_min  <- - outlier_max
ratio        <- 1.25
quan_num     <- 0.1
alpha        <- 0.3
prob_outlier <- 0.05 
KK           <- 3
K_L          <- 3
beta_index   <- 1
beta_number  <- 3

## if for scenario I
gene_para    <- set_genepara()

## if for scenario II
## gene_para    <- set_genepara(prob_outlier= prob_outlier,  outlier_max = outlier_max, outlier_min = outlier_min)


## I.2, generate data for scenario I
data       <- genedata_scenario_I(n1, gene_para$mu1_scenario_I , gene_para$mu0_scenario_I)
data_vad   <- genedata_scenario_I(n2, gene_para$mu1_scenario_I , gene_para$mu0_scenario_I)

## if generate data for scenario II
## data       <- genedata_scenario_II(n1, gene_para$prob_outlier, gene_para$outlier_max, 
   ##                                gene_para$outlier_min, gene_para$mu1_scenario_I, 
   ##                                 gene_para$mu0_scenario_I)
  
## data_vad   <- genedata_scenario_II(n2, gene_para$prob_outlier, gene_para$outlier_max, 
   ##                                gene_para$outlier_min, gene_para$mu1_scenario_I, 
   ##                                 gene_para$mu0_scenario_I)


## I.3, optimal rule for scenario I (not for scenario II)

beta_0 <- - 1
beta_1 <- 1/( 2 + sqrt(2) * qnorm(1 - alpha))
beta_2 <- beta_1
beta_3 <- 0


TP_0   <- 1 - pnorm(qnorm(1 - alpha) - sqrt(2))
FP_0   <- alpha 

## I.4, the standard approach

beta_mle               <- calpara_constraint_TPFP_mle(data, alpha, KK)

TPFP_diff_standard     <- caldiff_TPFP_linklinear(data_vad, beta_mle, 1)

## I.5, plug-in-lg

para_TPFP_constraint_plugin_lg    <- calpara_constraint_plugin(data, KK, alpha, fitting_method = "logistic")

TPFP_constraint_plugin_lg         <-   calTPFP_constraint_plugin(data, data_vad, para_TPFP_constraint_plugin_lg$lambda,
                                                                   para_TPFP_constraint_plugin_lg$theta, fitting_method = "logistic")
                                                                   

## I.6, plug-in-rf                                                        

para_TPFP_constraint_plugin_rf    <- calpara_constraint_plugin(data, KK, alpha, fitting_method = "randomforest")

TPFP_constraint_plugin_rf         <-   calTPFP_constraint_plugin(data, data_vad, para_TPFP_constraint_plugin_rf$lambda, 
                                                                   para_TPFP_constraint_plugin_rf$theta, fitting_method = "randomforest")

## I.7, plug-in-cv

meth                               <- calTPFP_constraint_plugin_choose(data, data_vad, KK, alpha, KK)

if (meth == "lg"){
  
  TPFP_constraint_plugin_TP       <- TPFP_constraint_plugin_lg$TP
  TPFP_constraint_plugin_FP       <- TPFP_constraint_plugin_lg$FP
  
  
} else if (meth == "rf"){
  
  TPFP_constraint_plugin_TP       <- TPFP_constraint_plugin_rf$TP
  TPFP_constraint_plugin_FP       <- TPFP_constraint_plugin_rf$FP
  
  
}


## I.8, DOSA with logistic loss surrogate, and with hinge loss surrogate

beta_removeoutlier     <- initial_remove_outlier(data, alpha, KK, quan_num)

temp_log               <- calpara_constraint_Subgradient_varystartpoint(data, as.matrix(beta_mle[-1])/abs(beta_mle[1]), as.matrix(beta_removeoutlier[-1])/abs(beta_removeoutlier[1]), 
                                                                   alpha, ratio,  n_S, a_S, lambda_low, lambda_up, grid_number, S_lambda, KK, K_L, beta_index, beta_number, 
                                                                   approx_method = "logistic")

temp_hin               <- calpara_constraint_Subgradient_varystartpoint(data, as.matrix(beta_mle[-1])/abs(beta_mle[1]), as.matrix(beta_removeoutlier[-1])/abs(beta_removeoutlier[1]), 
                                                                   alpha, ratio,  n_S, a_S, lambda_low, lambda_up, grid_number, S_lambda, KK, K_L, beta_index, beta_number, 
                                                                   approx_method = "hinge")

beta1_log              <- temp_log$beta1

beta0_log              <- temp_log$beta0

beta1_hin              <- temp_hin$beta1

beta0_hin              <- temp_hin$beta0

lambda_log             <- temp_log$lambda

lambda_hin             <- temp_hin$lambda

TPFP_diff_log          <- caldiff_TPFP_linklinear(data_vad, rbind(as.matrix(beta0_log), beta1_log), 1)

TPFP_diff_hin          <- caldiff_TPFP_linklinear(data_vad, rbind(as.matrix(beta0_hin), beta1_hin), 1)


## I.9, inference of DOSA for scenario I


V_log              <- cal_variance_parameter_constraint(data, alpha, beta0_log, beta1_log, lambda_log, b, approx_method = "logistic")

sd_log            <- diag(V_log)^0.5

V_hin             <- cal_variance_parameter_constraint(data, alpha, beta0_hin, beta1_hin, lambda_hin, b, approx_method = "hinge")

sd_hin            <- diag(V_hin)^0.5

pp                <- dim(data$Marker1)[2]

temp_log          <- cal_meanandsd_TP_constraint(data, alpha, beta0_log, beta1_log, lambda_log, b, delta_beta, approx_method = "logistic" ) 

temp_hin          <- cal_meanandsd_TP_constraint(data, alpha, beta0_hin, beta1_hin, lambda_hin, b, delta_beta, approx_method = "hinge" ) 

mu_TP_log <- temp_log$mean_TP
sd_TP_log <- temp_log$sd_TP
mu_FP_log <- 0
sd_FP_log <- 0

mu_TP_hin <- temp_hin$mean_TP
sd_TP_hin <- temp_hin$sd_TP
mu_FP_hin <- 0
sd_FP_hin <- 0


I_beta_1_log <- 0
I_beta_2_log <- 0
I_beta_3_log <- 0
I_TP_log     <- 0
I_FP_log      <- 0

I_beta_1_hin <- 0
I_beta_2_hin <- 0
I_beta_3_hin <- 0
I_TP_hin     <- 0
I_FP_hin     <- 0


if (beta1_log[1] - 1.96 *sd_log[1] <= beta_1 && beta_1 <= beta1_log[1] + 1.96 *sd_log[1]) { I_beta_1_log <- 1}
if (beta1_log[2] - 1.96 *sd_log[2] <= beta_2 && beta_2 <= beta1_log[2] + 1.96 *sd_log[2]) { I_beta_2_log <- 1}
if (beta1_log[3] - 1.96 *sd_log[3] <= beta_3 && beta_3 <= beta1_log[3] + 1.96 *sd_log[3]) { I_beta_3_log <- 1}
if (mu_TP_log - 1.96 *sd_TP_log <= TP_0 && TP_0 <= mu_TP_log + 1.96 *sd_TP_log) { I_TP_log <- 1}
if (mu_FP_log - 1.96 *sd_FP_log <= FP_0 && FP_0 <= mu_FP_log + 1.96 *sd_FP_log) { I_FP_log <- 1}

if (beta1_hin[1] - 1.96 *sd_hin[1] <= beta_1 && beta_1 <= beta1_hin[1] + 1.96 *sd_hin[1]) { I_beta_1_hin <- 1}
if (beta1_hin[2] - 1.96 *sd_hin[2] <= beta_2 && beta_2 <= beta1_hin[2] + 1.96 *sd_hin[2]) { I_beta_2_hin <- 1}
if (beta1_hin[3] - 1.96 *sd_hin[3] <= beta_3 && beta_3 <= beta1_hin[3] + 1.96 *sd_hin[3]) { I_beta_3_hin <- 1}
if (mu_TP_hin - 1.96 *sd_TP_hin <= TP_0 && TP_0 <= mu_TP_hin + 1.96 *sd_TP_hin) { I_TP_hin <- 1}
if (mu_FP_hin - 1.96 *sd_FP_hin <= FP_0 && FP_0 <= mu_FP_hin + 1.96 *sd_FP_hin) { I_FP_hin <- 1}




## I.10, rule out case
## The data can be changed to:

data$Y        <- 1 - data$Y
data$Marker   <- - data$Marker
data$Marker1  <- - data$Marker1
data$Marker[, 1]  <- -data$Marker[, 1]

data_vad$Y            <- 1 - data_vad$Y
data_vad$Marker       <- - data_vad$Marker
data_vad$Marker1      <- - data_vad$Marker1
data_vad$Marker[, 1]  <- -data_vad$Marker[, 1]

## Then the question TPF >= 1- gamma, min FPF
## so, it changes to FPF <= gamma, max TPF
